/*
 * $Id: UcpCallListRequests.java 1131 2015-02-11 10:55:53Z tpfeifer $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationCallList.java $
 * 
 * $LastChangedDate: 2015-02-11 11:55:53 +0100 (Wed, 11 Feb 2015) $ $LastChangedBy: tpfeifer $
 * $LastChangedRevision: 1131 $
 */
package de.starface.integration.uci.java.v30.ucp.messages.requests;

import java.util.Date;
import java.util.List;
import java.util.Map;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.values.CallListEntryDirection;
import de.starface.integration.uci.java.v30.values.CallListEntryProperties;
import de.starface.integration.uci.java.v30.values.CallListEntryResult;
import de.starface.integration.uci.java.v30.values.CallListProperties;
import de.starface.integration.uci.java.v30.values.CallState;
import de.starface.integration.uci.java.v30.values.GroupRestriction;
import de.starface.integration.uci.java.v30.values.OrderDirection;

/**
 * This interface defines the call list requests provided by the UCI server.
 * 
 * @author tilmann
 */
@UciService(UcpCallListRequests.SERVICE_NAME)
@RpcInterface(UcpCallListRequests.SERVICE_NAME)
public interface UcpCallListRequests
{
	
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.requests.callList";
	
	/**
	 * Retrieve a list of phone calls of the current user. The number of entries retrieved can be
	 * restricted by several parameters of this request. The results are returned as {@link Map}
	 * containing the values for the keys defined in {@link CallListProperties}.
	 * 
	 * @param startBefore
	 *        limit returned results to calls started before the given point of time
	 * @param startAfter
	 *        limit returned results to calls started after the given point of time
	 * @param directionRestriction
	 *        restrict returned results to calls of the given {@link CallListEntryDirection}. If an
	 *        empty String is given no restriction is used.
	 * @param resultRestriction
	 *        restrict returned results to calls of the given {@link CallListEntryResult}. If an
	 *        empty String is given no restriction is used.
	 * @param groupRestriction
	 *        restrict returned results to those conducted by a specified group. Possible choices
	 *        are the String {@link GroupRestriction#NON_GROUP} to return all calls that do not
	 *        belong to any group, the id of a group to restrict the list to the calls belonging to
	 *        that group and the empty String to return all calls without restriction.
	 * @param orderProperty
	 *        give the name of one of the {@link CallListEntryProperties} to order results by the
	 *        value of the property. Allowed values are {@link CallListEntryProperties#startTime},
	 *        {@link CallListEntryProperties#duration}, {@link CallListEntryProperties#callerNumber}
	 *        and {@link CallListEntryProperties#calledNumber}. When an empty String or invalid
	 *        property name is given the default {@link CallListEntryProperties#startTime} is used.
	 * @param orderDirection
	 *        give the name of one of the {@link OrderDirection}s to define the ordering. If not set
	 *        the default order direction of the property will be used
	 * @param countOffset
	 *        the offset of the first returned value in the result list after the ordering has been
	 *        applied
	 * @param countLimit
	 *        the count of items returned in the result list
	 * @return a {@link Map} containing the results with the properties defined in
	 *         {@link CallListProperties}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public Map<String, Object> getCallList(Date startBefore, Date startAfter, String directionRestriction,
										   String resultRestriction, String groupRestriction,
										   String orderProperty, String orderDirection, int countOffset,
										   int countLimit) throws UciException;
	
	/**
	 * Retrieve a list of redials of the current user. The results are returned as {@link List} of
	 * {@link Map} containing the values for the keys defined in {@link CallListEntryProperties}.
	 */
	public List<Map<String, Object>> getRedialList() throws UciException;
	
	/**
	 * Remove a call list entry from the list
	 * 
	 * @param callId
	 *        the id of the call list entry to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteCallListEntry(String callId) throws UciException;
	
	/**
	 * Remove multiple call list entries from the list
	 * 
	 * @param callIds
	 *        the ids of the call list entries to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteCallListEntries(List<String> callIds) throws UciException;
	
	/**
	 * This request is called to initiate a phone call for the user the same way as when using the
	 * STARFACE call list. The callListEntryId to call and phone to use can be selected using the
	 * request parameters. In UCI phone calls are identified using UUIDs (Universally Unique
	 * Identifier). The id for the call to be placed can either be given by the client or generated
	 * by STARFACE. The server may reject the request to place the call due to another place call
	 * request being in progress. If the result of the message is a valid UUID the call request is
	 * accepted and the call is in the state {@link CallState#OUTGOING}.
	 * 
	 * @param callListEntryId
	 *        The identification of the call list entry.
	 * @param phoneId
	 *        The id of the telephone used to make the phone call. If not set the primary phone will
	 *        be used.
	 * @param callId
	 *        The UUID to be assigned to the phone call. If not set STARFACE will create one.
	 * @return an empty string if the place call request was rejected by the server or the UUID
	 *         assigned to the call otherwise
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String placeCallbackCallWithPhone(String callListEntryId, String phoneId, String callId)
		throws UciException;
	
}
